@@ -1,3 +1,5 @@
+from typing import Iterable
+
 import ipywidgets as widgets
 from IPython.display import display
 
@@ -5,8 +7,9 @@ from IPython.display import display
 def DEFINE(  # pylint: disable=invalid-name
         name: str,
         default,
-        options: list = [],
+        options: Iterable = (),
         type: str = '',
+        multiline: bool = False,
         **args):
     """Registers a generic Flag object.
 
@@ -17,44 +20,39 @@ def DEFINE(  # pylint: disable=invalid-name
     function instead.
 
     Args:
-      parser: ArgumentParser, used to parse the flag arguments.
       name: str, the flag name.
       default: The default value of the flag.
-      help: str, the help message.
-      flag_values: FlagValues, the FlagValues instance with which the flag will be
-        registered. This should almost never need to be overridden.
-      serializer: ArgumentSerializer, the flag serializer instance.
-      module_name: str, the name of the Python module declaring this flag. If not
-        provided, it will be computed using the stack trace of this call.
       **args: dict, the extra keyword args that are passed to Flag __init__.
-
-    Returns:
-      a handle to defined flag.
     """
-    _DEFINE_param(True, name, default, options, type)
+    _DEFINE_param(True, name, default, options, type, multiline)
 
 
-def _DEFINE_param(universal: bool, name: str, default, options: list = [], type: str = ''):
-    if universal and type == '':
+def _DEFINE_param(universal: bool, name: str, default, options: Iterable = (), type: str = '', multiline: bool = False):
+    if universal and not type:
+        if multiline:
+            type = 'Textarea'
         if isinstance(default, int):
             type = 'IntText'
         if isinstance(default, float):
             type = 'FloatText'
         if isinstance(default, bool):
             type = 'Checkbox'
-        if isinstance(default, list) and default and not options:
+        if isinstance(default, Iterable) and default and not options:
             options = default
-            default = options[0]
+            default = next(iter(options))
         if options:
             type = 'Select'
-        type = 'Text' if type == '' else type
+        type = type if type else 'Text'
 
     setattr(PARAMS, name, default)
 
     if type == 'Select':
-        widget = widgets.Dropdown(value=default, description=name, options=options)
+        widget = widgets.Dropdown(value=default, description=name, options=[*options])
+    elif type == 'SelectMultiple':
+        widget = widgets.SelectMultiple(value=default, description=name, options=[*options])
     else:
         widget = eval('widgets.{}(value=default, description=name)'.format(type))
+    PARAMS.widgets[name] = widget
 
     def on_value_change(change):
         setattr(PARAMS, name, change['new'])
@@ -64,8 +62,16 @@ def _DEFINE_param(universal: bool, name: str, default, options: list = [], type:
     display(widget)
 
 
-def DEFINE_string(name: str, default, help: str = 'for compatibility with absl.flags only', **args):
-    _DEFINE_param(False, name, default, type='Text')
+def DEFINE_string(name: str, default, help: str = 'for compatibility with absl.flags only', multiline: bool = False,
+                  **args):
+    if multiline:
+        _DEFINE_param(False, name, default, type='Textarea')
+    else:
+        _DEFINE_param(False, name, default, type='Text')
+
+
+def DEFINE_text(name: str, default, help: str = 'for compatibility with absl.flags only', **args):
+    DEFINE_string(name, default, multiline=True)
 
 
 def DEFINE_boolean(name: str, default, help: str = 'for compatibility with absl.flags only', **args):
@@ -80,20 +86,27 @@ def DEFINE_integer(name: str, default, help: str = 'for compatibility with absl.
     _DEFINE_param(False, name, default, type='IntText')
 
 
-def DEFINE_enum(name: str, default, options: list = [], help: str = 'for compatibility with absl.flags only', **args):
-    if isinstance(default, list) and default and not options:
+def DEFINE_enum(name: str, default, options: Iterable = (), help: str = 'for compatibility with absl.flags only',
+                **args):
+    if isinstance(default, Iterable) and default and not options:
         options = default
-        default = options[0]
+        default = next(iter(options))
     _DEFINE_param(False, name, default, options, type='Select')
 
 
-def DEFINE_multi_enum(name: str, default, options: list = [], help: str = 'for compatibility with absl.flags only', **args):
-    if not isinstance(default, list):
+def DEFINE_multi_enum(name: str, default, options: Iterable = (), help: str = 'for compatibility with absl.flags only',
+                      **args):
+    if isinstance(default, Iterable) and default and not options:
+        options = default
+        default = next(iter(options))
+    if not isinstance(default, Iterable):
         default = [default]
     _DEFINE_param(False, name, default, options, type='SelectMultiple')
 
 
 class PARAMS(object):
+    widgets = {}
+
     @classmethod
     def set_default(cls, name, value):
         """Changes the default value of the named flag object.
@@ -106,4 +119,5 @@ class PARAMS(object):
           name: str, the name of the flag to modify.
           value: The new default value.
         """
+        cls.widgets[name].value = value
         setattr(cls, name, value)
